import { globalLookup, globalSetting, signalValues } from "./global";

let mainRenderEl = null;
const canvasMap = {};

/**
 * 
 * @returns {Element}
 */
function getMainRenderEl() {
    if (!mainRenderEl) {
        const el = document.querySelectorAll('.vcd-render-wrapper')[0];
        if (el) {
            mainRenderEl = el;
        }
    }
    return mainRenderEl;
}

/**
 * 
 * @param {{
 *  time: BigInt,
 *  cmd: number,
 *  value?: BigInt,
 *  mask?: BigInt
 * }} dataItem 
 * @returns 
 */
function isBlocked(dataItem) {
    const mask = dataItem.mask;
    if (mask && mask === 1n) {
        return true;
    }
    if (mask && mask === 0n) {
        return false;
    }
    if (mask === undefined && dataItem.cmd > 15) {
        return true;
    }
    return false;
}

/**
 * @param {{
 * kind: 'var' | 'scope',
 * type: 'wire' | 'reg' | 'module',
 * name: string,
 * size: number,
 * link: string
 * }} signal 
 */
function makeWaveCanvas(signal) {
    const mainRender = getMainRenderEl();
    const canvas = document.createElement('canvas');
    canvas.id = 'wave-' + signal.link;
    canvas.className = 'render-canvas';
    canvas.width = document.body.clientWidth;
    canvas.height = globalSetting.displaySignalHeight;
    
    mainRender.appendChild(canvas);
    canvasMap[signal] = canvas;

    const ctx = canvas.getContext('2d');
    const datas = signalValues[signal.link];

    if (signal.size === 1) {
        drawSimpleSignal(ctx, datas);
    } else {
        // adwa
    }
}

/**
 * @param {{
* kind: 'var' | 'scope',
* type: 'wire' | 'reg' | 'module',
* name: string,
* size: number,
* link: string
* }} signal
*/
function removeWaveCanvas(signal) {
    const canvas = canvasMap[signal];
    if (canvas instanceof HTMLCanvasElement) {
        canvas.parentNode.removeChild(canvas);
    }
}


/**
 * 
 * @param {CanvasRenderingContext2D} ctx 
 * @param {Array<{
 *  time: BigInt,
 *  cmd: number,
 *  value?: BigInt,
 *  mask?: BigInt
 * }>} signals 
 */
function drawSimpleSignal(ctx, signals) {
    let last_data = null;
    let last_x = 300;
    ctx.lineWidth = 3;
    ctx.lineJoin = 'round';
    
    ctx.beginPath();
    for (const data of signals) {
        console.log(data);
        if (last_data === null) {
            last_data = data;
            last_x = 300;
            let last_y = Math.max((15 - last_data.cmd) * 30, 0) + 10;
            ctx.moveTo(last_x, last_y);
        } else {
            // 高阻态
            const start = last_data.time;
            const end = data.time;
            ctx.strokeStyle = isBlocked(last_data) ? 'rgb(224,108,117)' : 'rgb(10, 200, 10)';

            let last_y = Math.max((15 - last_data.cmd) * 30, 0) + 10;

            const duration = parseInt(end - start);
            const current_x = last_x + 100;

            console.log(current_x, last_y, duration);
            ctx.lineTo(current_x, last_y);

            if (last_data.cmd !== data.cmd) {
                ctx.strokeStyle = isBlocked(data) ? 'rgb(224,108,117)' : 'rgb(10, 200, 10)';
                let current_y = Math.max((15 - data.cmd) * 30, 0) + 10;
                ctx.lineTo(current_x, current_y);
            }

            last_data = data;
            last_x = current_x;
        }
    }
    ctx.stroke();
}

export {
    getMainRenderEl,
    makeWaveCanvas,
    removeWaveCanvas
}